#define _XOPEN_SOURCE 700 // for sigaction...

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dlfcn.h>
#include <memory.h>
#include <signal.h>

#include "foo.h"

static sig_atomic_t reload = 0;

static void sigusr1_handler(int sig)
{
  reload = 1;
}

static const char* fooPath = "./libfoo.so";

struct fooStruct {
  void *handle;
  struct foo_api api;
  struct data_t *data;
};

void loadFoo(struct fooStruct *foo)
{
  if (foo->handle) {
    dlclose(foo->handle);
    foo->handle = NULL;
  }
  dlerror();
  foo->handle = dlopen(fooPath, RTLD_NOW);
  if (foo->handle == NULL) {
    printf("%s\n",dlerror());
  }
  const struct foo_api *api = dlsym(foo->handle, "FOO_API");
  if (api != NULL) {
    foo->api = *api;
  } else {
    printf("error: %s\n", dlerror());
    foo->handle = NULL;
  }
}

int main(int argc, char *argv[])
{
#ifdef RELOAD_SUPPORT
  struct sigaction act;
  memset(&act, 0, sizeof(act));
  act.sa_handler = sigusr1_handler;
  sigemptyset(&act.sa_mask);
  sigaction(SIGUSR1, &act, NULL);
#endif

  struct fooStruct foo = { 0 };
  loadFoo(&foo);

  foo.data = foo.api.init();

  for (;;) {
    if (foo.handle != NULL) {
#ifdef RELOAD_SUPPORT
      sleep(1);
      if (reload == 1) {
        printf("Current string from foo: %s\n", foo.api.text());
        printf("Reloading...\n");
        printf("Setting foo.data->counter to %d\n", 15);
        foo.api.setC(foo.data, 15);
        loadFoo(&foo);
        reload = 0;
      }
#endif
      if (foo.api.work(foo.data) == -1) { 
        break;
      }
    }
  }
  free(foo.data);
  return 0;
}
